;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Retro Tennis Super Wave 
;;  Copyright (C) 2019 Bamboo Studio 
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  Bamboo Studio.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see http://www.gnu.org/licenses/.
;;------------------------------------------------------------------------------
.include "cpctelera.h.s"
.include "sys/physics.h.s"
.include "sys/render_system.h.s"
.include "sys/input.h.s"
.include "entity_manager.h.s"
.include "sys/ai_control.h.s"
.include "sys/state.h.s"

JUEGO_TERMINADO == 0
_partido: .db 0

;.module game_manager

DefineEntity player, 38, 150, 1, 2, 10, 40, _sp_player, e_ai_st_noAI
DefineEntity enemy, 38, 30, -2, 0, 10, 40, _sp_enemy1, e_ai_st_move_to
DefineEntity ball, 36, 120, 0, 1, 2, 8, _sp_ball, e_ai_st_noAI ;;vx 0 vy 
DefineSign marc_pl, 10, 10, 15, 16, _sp_marc_00
DefineSign marc_cpu, 55, 10, 15, 16, _sp_marc_00
DefineSign easter_egg, 35, 50, 14, 32, _sp_ee

man_game_init::
   

   ld a, #0
   ld (JUEGO_TERMINADO), a

   ;;Init systems
   call rendersys_init

   ;;Create entities
      ld    hl, #player
      call entityman_create

      ld    hl, #enemy
      call entityman_create

      ld    hl, #ball
      call entityman_create

      ld    hl, #marc_pl
      call entityman_sign_create

      ld    hl, #marc_cpu
      call entityman_sign_create

      ld    hl, #easter_egg
      call entityman_sign_create

      call update_sign_player
      call update_sign_cpu

      call entityman_getSignArray
      call rendersys_render_signs

      call entityman_getEntityArray
      call sys_ai_control_init

      call sys_state_init

   ;Draw entities for the first time before starting erasing (drawing again with blending)
   call entityman_getEntityArray
   ld (_first_ent_counter), a

   _loopfirstren:
      ld   e, e_lastVP_l(ix) ;Pointer to last position
      ld   d, e_lastVP_h(ix)
      ld l,  e_sp_l(ix) ;; Pointer to sprite low byte
      ld h,  e_sp_h(ix) ;; Pointer to sprite high byte
      ld   c, e_h(ix) ;Height
      ld   b, e_w(ix) ;Width
      call cpct_drawSpriteBlended_asm ;Draw the solid box to erase the sprite


   _first_ent_counter = .+1
      ld  a, #0
      dec a
      ret z ;When all the entities are drawn once then return if not then we'll go on to the next entity

      ld (_first_ent_counter), a
      ld bc, #sizeof_e
      add ix, bc

      jr _loopfirstren

   ;ret

man_game_update::

   call entityman_getEntityArray
   call  sys_input_update

   call entityman_getEntityArray
   call sys_ai_control_update

   call entityman_getEntityArray
   call  sys_physics_update

   call entityman_getEntityArray_Collisions
   call  sys_physics_collisions

   call updatePathClock


   call sys_state_load
   inc a
   dec a
   jr z, _looppaused

   ret

   _looppaused:
   call sys_state_init
   call man_game_reset

   ret

man_game_render::
   call entityman_getEntityArray
   call rendersys_update

   ret


man_game_reset::
   
   call entityman_getEntityArray

   ld b, #100

   _looptime:

   halt
   dec b

   jr nz, _looptime

   ld e_x(ix), #38
   ld e_y(ix), #150
   ld e_vx(ix), #0
   ld e_vy(ix), #0

   ld de, #sizeof_e
   add ix, de

   ld e_x(ix), #38
   ld e_y(ix), #30
   ld e_vx(ix), #1
   ld e_vy(ix), #0

   ld de, #sizeof_e
   add ix, de

   ld e_x(ix), #36
   ld e_y(ix), #120
   ld e_vx(ix), #0
   ld e_vy(ix), #1

   call sys_physics_reset_bounce ;Se resetean las variables del bote

    push ix
    call cpct_akp_stop_asm ;Se para la musica actual en los 3 canales
    pop ix

   ld a, (_score_player) ;Comprobar si es victoria
   cp #4
   jr z, _loopfin_v

   ld a, (_score_cpu) ;Comprobar si es derrota
   cp #4
   jr z, _loopfin_d

   ret ;Si no se vuelve y se continua el partido

   _loopfin_v:

   ld    a, (_partido)
   inc   a
   ld    (_partido), a

   call  man_game_win

   ret

   _loopfin_d:

   ld    a, #0
   ld    (_partido), a

   call man_game_lose

   ret

man_game_win::

   ld a, #1
   ld (JUEGO_TERMINADO), a

   ld a, #0
   ld (_score_player), a ;Se resetean las puntuaciones del jugador y la CPU
   ld (_score_cpu), a 

   push hl
   call entityman_sign_reset
   call entityman_reset
   pop hl

   call mostrar_victoria
   ret

man_game_lose::
   ld a, #1
   ld (JUEGO_TERMINADO), a

   ld b, #100

   ;MOSTRAR SPRITE DEL EASTER EGG
   call entityman_getSignArray
   call rendersys_render_ee

   _loopee:

   halt   
   djnz _loopee ;Decrease b jump no z 

   ld a, #0
   ld (_score_player), a ;Se resetean las puntuaciones del jugador y la CPU
   ld (_score_cpu), a 

   push hl
   call entityman_sign_reset
   call entityman_reset
   pop hl

   call mostrar_derrota

   ret
