;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Retro Tennis Super Wave 
;;  Copyright (C) 2019 Bamboo Studio 
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  Bamboo Studio.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see http://www.gnu.org/licenses/.
;;------------------------------------------------------------------------------
;;
;;	ENTITY AI-CONTROL SYSTEM
;;
.include "man/entity_manager.h.s"
.include "cpctelera.h.s"

_tracker: .db 00

.module sys_ai_control

;;========================================================================
;;  Physics System Constants
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  SYS_AI_CONTROL::Init
;;		Inits the AI-control system
;;	INPUT
;;		IX = Pointer to the entity array

sys_ai_control_init::
	;;; !!!! TODO: Remove: Temporal!!!!
	ld (_ent_array_ptr_temp_standby), ix ;; [6]
	ld (_ent_array_ptr), ix ;; [6]
	ld a, #2
	ld (_tracker), a
	ret	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	SYS_AI_CONTROL::Update
;;		Update all AI entities
;;	INPUT
;;		IX = Pointer to the entity array
;;		 A = Number of elements in the array
;;	DESTROYS: AF, BC, DE, IX
;;  STACK USE: 2 bytes

sys_ai_stand_by::
_ent_array_ptr_temp_standby = .+2
	ld	iy, #0x0000	;;[4]

	ld  a, e_ai_aim_x(iy)
	or  a
	ret z

	ld     e_ai_st(ix), #e_ai_st_move_to

;; Pressed key. Move_to ball 

	ret

sys_ai_move_to::

	ld   a, e_vy(iy)
	cp   #-1
	jr   z, _back

_go:

	ld   a, e_y(iy)
	sub  #95

	jr   nc, _norun

	ld   a, #1
	ld   e_vy(ix), a

	jr   _continue

_back:

	ld   a, e_y(iy)
	sub  #95

	jr   nc, _norun

	ld   a, #-1
	ld   e_vy(ix), a

	jr   _continue

_norun:

	ld   a, #0
	ld   e_vy(ix), a

_continue:

	ld  a, (_tracker)
	dec a
	ld  (_tracker), a
	cp  #0
	jr  nz, _notrack

	ld a, (_partido)
	cp #0
	jr z, _IA_0

	ld a, (_partido)
	cp #1
	jr z, _IA_1

	ld a, (_partido)
	cp #2
	jr z, _IA_2

	ret

.macro saveSetBorder _color
	push bc
	cpctm_setBorder_asm _color
	pop bc
.endm

_IA_0:
	saveSetBorder HW_GREEN

	ld	a, e_x(iy)
	ld  e_ai_aim_x(ix), a
	ld  a, #50 ;; VARIABLE CICLOS DE TRACKING
	dec a
	ld  (_tracker), a
	jr _notrack

_IA_1:
	saveSetBorder HW_YELLOW

	ld	a, e_x(iy)
	ld  e_ai_aim_x(ix), a
	ld  a, #30 ;; VARIABLE CICLOS DE TRACKING
	dec a
	ld  (_tracker), a
	jr _notrack

_IA_2:
	saveSetBorder HW_RED

	ld	a, e_x(iy)
	ld  e_ai_aim_x(ix), a
	ld  a, #10 ;; VARIABLE CICLOS DE TRACKING
	dec a
	ld  (_tracker), a
	jr _notrack

_notrack:

	ld	 a, e_ai_aim_x(ix)   ;; A = x_objetivo
	sub  e_x(ix)			 ;; A = obj_x - x
	jr   nc, _objx_greater_or_equal
_objx_lesser:
	ld   e_vx(ix), #-1
	jr	 _endif_x
_objx_greater_or_equal:
	jr   z, _equal
	ld   e_vx(ix), #1
	jr	 _endif_x
_equal:
	ld   e_vx(ix), #0
	jr	 _endif_x
_endif_x:

ret

sys_ai_control_update::
	ld	(_ent_counter), a

_ent_array_ptr = .+2
	ld ix, #0x0000 ;;[4]

_loop:

	ld	a, e_ai_st(ix)
	cp  #e_ai_st_noAI
	jr  z, _no_AI_ent
_AI_ent:
	cp    #e_ai_st_stand_by
	call  z, sys_ai_stand_by
	cp    #e_ai_st_move_to
	call  z, sys_ai_move_to
_no_AI_ent:

_ent_counter = . +1
	ld   a, #0
	dec	 a
	ret  z

	ld	 (_ent_counter), a
	;;   Jump to next entity
	ld	 de, #sizeof_e
	add  ix, de

	jr	_loop