;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Retro Tennis Super Wave 
;;  Copyright (C) 2019 Bamboo Studio 
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  Bamboo Studio.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see http://www.gnu.org/licenses/.
;;------------------------------------------------------------------------------
.include "man/entity_manager.h.s"
.include "state.h.s"
.include "cpctelera.h.s"
.include "cpct_functions.h.s"	

.module sys_entity_physics

_last_pos: .dw 00 
_cont_iy: .db 0 
_status_ball: .db 4 ;;Contador de la bola para el punto
_cont_bounce: .db 1 ;;Contador de la bola para el bote (giro)
_cont: .db 95
_clock_hit: .db 50
_id_shot: .db 0 ;Id del ultimo que golpeo la pelota 1=player 2=cpu
_update_x: .db 0
_vel_transmit: .db 0
_score_player: .db 0
_score_cpu: .db 0

screen_width = 80
screen_height = 200
calle_width_0 = 15
calle_width = screen_width - calle_width_0


sys_physics_init::
	ret
 
sys_physics_update::
	ld 		b, a

_update_loop:
	
	ld	c, b
	ld	a, c
	cp	#1
	jr	nz, lim_players ;Si no es la pelota entonces se comprobaran los limites de los jugadores

	call sys_boundaries_ball ;Comprobaciones de punto de la pelota

	jr	endif_y ;Me voy al final porque las comprobaciones de la pelota son otras

;;UPDATEAR LA X

lim_players:

	ld		a, #screen_width + 1
	sub 	e_w(ix)
	ld 		c, a

	ld 		a, e_x(ix)
	add 	e_vx(ix)
	cp 		c
	jr 		nc, invalid_x


valid_x:
	ld 		e_x(ix), a
	jr 		endif_x

invalid_x:
	ld 		a, e_vx(ix)
	neg
	ld 		e_vx(ix), a

endif_x:

;;UPDATEAR LA Y

	ld		a, #screen_height + 1
	sub 	e_h(ix)
	ld 		c, a

	ld 		a, e_y(ix)
	add 	e_vy(ix)
	cp 		c
	jr 		nc, invalid_y


valid_y:
	ld 		e_y(ix), a
	jr 		endif_y

invalid_y:
	ld 		a, e_vy(ix)
	neg
	ld 		e_vy(ix), a

endif_y:

	dec 	b
	ret 	z


	ld		de, #sizeof_e
	add 		ix, de
	jr 		_update_loop

sys_boundaries_ball::

	ld		a, #calle_width + 1
	sub 	e_w(ix)
	ld 		c, a

	ld 		a, e_x(ix)
	add 	e_vx(ix)
	cp 		c
	jr 		nc, point_calle

; 	ld		a, e_x(ix)
; 	cp		#calle_width
; 	jr 		z, point_calle

; 	ld		a, e_x(ix)
; 	cp		#calle_width_0
; 	jr 		z, point_calle

ball_valid_x:
	ld 		e_x(ix), a
	jr 		ball_endif_x

point_calle:
	call sys_state_pause
				;ORDEN CAMBIADO PORQUE LA CALLE ES UNA ZONA ILEGAL ASI QUE SE SANCIONA
	ld	a, (_id_shot) ;Ahora incrementaremos los puntos de uno u otro jugador segun a quien corresponda el punto 
	dec	a
	jr	z, point_calle_CPU ;Si era un tiro del player entonces sera punto para la CPU

	ld	a, (_score_player) ;Si no sera un punto para el player
	inc	a
	ld	(_score_player), a
	;AQUI ACTUALIZO EL SPRITE DEL SCORE DEL PLAYER	
	push bc
	push ix
	push iy
	call update_sign_player
	pop iy
	pop ix
	pop bc

	jr	point_calle_end

	point_calle_CPU:
	ld	a, (_score_cpu)
	inc	a
	ld	(_score_cpu), a
	;AQUI ACTUALIZO EL SPRITE DEL SCORE DE LA CPU
	push bc
	push ix
	push iy
	call update_sign_cpu
	pop iy
	pop ix
	pop bc

	point_calle_end:

ball_endif_x:

;;UPDATEAR LA Y

	ld		a, #screen_height + 1
	sub 	e_h(ix)
	ld 		c, a

	ld 		a, e_y(ix)
	add 	e_vy(ix)
	cp 		c
	jr 		nc, point_limy


ball_valid_y:
	ld 		e_y(ix), a
	jr 		ball_endif_y

point_limy:
	call sys_state_pause

	ld	a, (_id_shot) ;Ahora incrementaremos los puntos de uno u otro jugador segun a quien corresponda el punto
	dec	a
	jr	z, point_limy_player ;Si era un tiro del player entonces sera punto para el player

	ld	a, (_score_cpu) ;Si no sera un punto para la CPU
	inc	a
	ld	(_score_cpu), a
	;AQUI ACTUALIZO EL SPRITE DEL SCORE DE LA CPU	
	push bc
	push ix
	push iy
	call update_sign_cpu
	pop iy
	pop ix
	pop bc

	jr	point_limy_end

	point_limy_player:
	ld	a, (_score_player)
	inc	a
	ld	(_score_player), a
	;AQUI ACTUALIZO EL SPRITE DEL SCORE DEL PLAYER
	push bc
	push ix
	push iy
	call update_sign_player
	pop iy
	pop ix
	pop bc

	point_limy_end:

ball_endif_y:

	ret


sys_physics_collisions::
	ld 		b, a ;b = 3 (num_entities)


_next_ix:
	;Next base entity
	dec b 
	jp  z, _exit

	ld	(_last_pos), ix
	ld	iy, (_last_pos)
	ld	c, b ;b+1
	inc	c
	ld	a, c
	ld	(_cont_iy), a
_next_iy:
	;Next entity to compare collider with base entity
	;When the number of entities to compare is 0 then we go on to the next base entity
	ld	a, (_cont_iy)
	dec 	a
	ld	(_cont_iy), a
	jp 	z, _trozo

	ld		de, #sizeof_e
	add 		iy, de

	;If the entities are the player and the ball, we are going to check if the space is also pressed
	ld c, b
	ld a, c
	cp #2 ;If entity base is not the player
	jr nz, otherCases

	ld	a, (_cont_iy) ;If the other entity is not the ball
	dec 	a
	jr nz, otherCases

	push bc
	call 	cpct_scanKeyboard_f_asm
	ld		hl, #Key_Space
	call	cpct_isKeyPressed_asm 
	pop bc
	jr		nz, soloHitPlayer ;If we press space with player and ball entities then we jump to soloHitPlayer, if not we jump to _no_collision

	;NO COLLISION BETWEEN PLAYER AND BALL
	jr		 _no_collision

	otherCases:
	;If the entities are the CPU and the ball, we are going to jump to soloHitCPU
	ld c, b
	ld a, c
	cp #1 ;If entity base is not the CPU then we jump to normalCollision
	jr nz, normalCollision


	soloHitCPU:
		;Calls to check colition
		call checkCollision 
	jr		c, _no_collision
	jr		_hit_ball_CPU	    

	soloHitPlayer:
		;Calls to check colition
		call checkCollision 
	jr		c, _no_collision
	jr		_hit_ball_Player


	normalCollision:
		;Calls to check colition
		call checkCollision 

	jr		c, _no_collision


_collision:
	;Exists collision so ball changes velocity

    ld		a, e_vy(iy)
    neg
    ld		e_vy(iy), a   

    	jr         _next_iy ;VA AQUI UN RET?

_hit_ball_CPU:

    ld		a, (_clock_hit)
    cp		#50
    jr		z, can_bounce_CPU

    jr		cant_bounce_CPU

    can_bounce_CPU:
    ld		a, #0 
    ld		(_clock_hit), a ;Ball bounced so we reset the hit clock 
    ;push bc
    call hit_cpu ;HABRA QUE HACER UN PUSH Y POP DE BC PARA NO CHAFAR EL REGISTRO EN ESTE BUCLE QUE LO USA PARA IX E IY??
    ;pop  bc

    cant_bounce_CPU:

    	jr         _next_iy

	ret

_hit_ball_Player:

    ld		a, (_clock_hit)
    cp		#50
    jr		z, can_bounce

    jr		cant_bounce

    can_bounce:
    ld		a, #0 
    ld		(_clock_hit), a ;Ball bounced so we reset the hit clock 
    ;push bc
    call hit_player ;HABRA QUE HACER UN PUSH Y POP DE BC PARA NO CHAFAR EL REGISTRO EN ESTE BUCLE QUE LO USA PARA IX E IY??
    ;pop  bc

    cant_bounce:

    	jp         _next_iy

	ret		
_no_collision:
    	jp         _next_iy

	ret
_trozo:
	ld		de, #sizeof_e
	add 		ix, de
	jp	_next_ix
_exit:
	ret


;;
;;CALCULAR COLISIONES
;;
checkCollision::
	ld      a, e_x(ix)
	add     e_w(ix)
	sub     e_x(iy)
	ret     c

	ld      a, e_x(iy)
	add     e_w(iy)
	sub     e_x(ix)
	ret     c

	ld      a, e_y(ix)
	add     e_h(ix)
	sub     e_y(iy)
	ret     c

	ld      a, e_y(iy)
	add     e_h(iy)
	sub     e_y(ix)

	ret

hit_player::

	ld	a, (_id_shot)
	cp	#2		;Si le ha dado la CPU entonces se inicia un nuevo ciclo en el bote
	jr	nz, _just_hit_player

	ld	a, #4 ;Restart of status ball, restart of cycle cause a new player hitted the ball
  	ld	(_status_ball), a

  	ld 	a, #95 ;Restart of cont
 	ld 	(_cont), a

 	ld 	a, #1
  	ld 	(_cont_bounce), a ;Restart of cont bounce

_just_hit_player:
	ld	a, #1 ;Se le pone la nueva id del golpe
	ld	(_id_shot), a

	ld	a, e_vx(ix) ;Me guardo la velocidad en x de la entidad que golpea y que transmitira a la pelota
	ld	(_vel_transmit), a

	call	bounce

	ret

hit_cpu::

	ld	a, (_id_shot)
	cp	#1		;Si le ha dado el player entonces se inicia un nuevo ciclo en el bote
	jr	nz, _just_hit_CPU

	ld	a, #4 ;Restart of status ball, restart of cycle cause a new player hitted the ball
  	ld	(_status_ball), a

  	ld 	a, #95 ;Restart of cont
 	ld 	(_cont), a

 	ld 	a, #1
  	ld 	(_cont_bounce), a ;Restart of cont bounce

_just_hit_CPU:
	ld	a, #2 ;Se le pone la nueva id del golpe
	ld	(_id_shot), a

	ld	a, e_vx(ix) ;Me guardo la velocidad en x de la entidad que golpea y que transmitira a la pelota
	ld	(_vel_transmit), a

	call	bounce

	ret

 bounce::
 	;COMPROBACIONES DEL STATUS BALL
 	ld	a, (_status_ball) ;If status ball is 4 then it'll start all over again, if not init_cycle won't be executed
 	cp	#4
 	jr	nz, cycle

  	first_bounce:
  	ld  a, (_cont)
  	cp  #95
  	jr  nz, cycle ;Compruebo si va a empezar el tiro, si lo estoy continuando solo entrare a cycle
  	
 	init_cycle:
  	;Se inicializa un nuevo golpe, golpe en sentido contrario 

  	ld	a, (_id_shot)
  	dec	a
  	jr	z, y_sentido_player

  	ld	e_vy(iy), #1

  	jr	cycle

  	y_sentido_player:

  	ld	e_vy(iy), #-1

 cycle:

  	;AND THEN EVERYTHING REPEATS ITSELF AGAIN

  	ld 	a, (_cont)
  	dec	a
  	ld	(_cont), a
  	ret   nz	;Una vez se me acabe el contador de este estado entonces cambiare de estado, mientras tanto seguire restando cont volviendo ahora con el ret

  	change_stat:
  	ld 		a, e_vy(iy)
  	neg
 	ld 		e_vy(iy), a

  	ld 	a, (_status_ball)
  	dec	a
  	ld 	(_status_ball), a

  	ld	a, (_cont_bounce)	;NUEVO VALOR DE CONT Y DE COUNT_BOUNCE
  	dec	a ;Relleno el nuevo cont segun corresponda con el nuevo valor de status_ball, primero compruebo el valor 1 que es con el que se inicializa
  	jr	z, shorter_cont ; Si es 1 entonces se hara el cont corto de 10 si es 0 entonces el largo de 95

  	ld 	a, #95
 	ld 	(_cont), a
 	jr	longer_cont

  	shorter_cont:
  	ld  a, #10
  	ld  (_cont), a

  	longer_cont:
  	
  	ld 	a, (_cont_bounce)
  	cp 	#0
  	jr 	z, reset_bounce

  	dec 	a
  	ld 	(_cont_bounce), a 

  	jr    continue_bounce

  	reset_bounce:

  	ld 	a, #1
  	ld 	(_cont_bounce), a 

  	continue_bounce:	;COMPROBACION DE PUNTO

  	ld 	a, (_status_ball) 
  	cp	#0 ;If the updated value of status ball is 0 then we'll go to point

  	ret   nz ;If the state is not 0 yet then we'll continue doing the same

  	point:

  	call sys_physics_reset_bounce ;Se resetean las variables del bote

	call sys_state_pause ;Se mete un punto

	ld	a, (_id_shot) ;Ahora incrementaremos los puntos de uno u otro jugador segun a quien corresponda el punto
	dec	a
	jr	z, point_player ;Si era un tiro del player entonces sera punto para el player

	ld	a, (_score_cpu) ;Si no sera un punto para la CPU
	inc	a
	ld	(_score_cpu), a
	;AQUI ACTUALIZO EL SPRITE DEL SCORE DE LA CPU	
	push bc
	push ix
	push iy
	call update_sign_cpu
	pop iy
	pop ix
	pop bc

	jr	point_end

	point_player:
	ld	a, (_score_player)
	inc	a
	ld	(_score_player), a
	;AQUI ACTUALIZO EL SPRITE DEL SCORE DEL PLAYER
	push bc
	push ix
	push iy
	call update_sign_player
	pop iy
	pop ix
	pop bc

	point_end:	

 	ret

 updatePathClock::
 	;Aqui solo actualizamos el reloj y llamamos a bounce para actualizar la trayectoria cuando sea necesario
 	ld		a, (_clock_hit)
    	cp		#50		
    	jr		z, updatePath ;Cuando sea distinto de 50 entonces se incrementara el reloj

    	ld		a, (_clock_hit) ;Increasing the hit clock and continuing
    	inc		a
    	ld		(_clock_hit), a

    	updatePath:
    	ld		a, (_id_shot) ;Si esta en medio de un golpe entonces se actualizara la trayectoria de la bola
    	cp		#0 ;Es decir que la id del shot debe ser o 1 o 2, si es 0, no se actualizara
    	jr		z, noUpdate

    	ld	a, (_update_x)
    	cp	#0
    	jr	nz, dec_x

    	ld	a, #3
    	ld	(_update_x), a ;Reseteamos el contador
    	ld	a, #0
    	ld	e_vx(iy), a ;Actualizamos la velocidad a 0 porque no debe avanzar en este ciclo

    	dec_x:
    	ld	a, (_update_x)
    	dec	a
    	ld	(_update_x), a

    	cp	#0
    	jr	nz, update_y

    	ld	a, (_vel_transmit) ;ACTUALIZAR X HACIA LA DERECHA, IZQ O CENTRO SEGUN LA VELOCIDAD DE LA ENTIDAD QUE GOLPEA A LA PELOTA
    	ld	e_vx(iy), a ;Actualizamos la velocidad porque debe avanzar en este ciclo

    	update_y:

    	call bounce

    	noUpdate:

 	ret

sys_physics_reset_bounce::

		ld	a, #4 ;Cycle finished, every status done, restart of status ball
		ld	(_status_ball), a

		ld 	a, #95 ;Restart of cont
		ld 	(_cont), a

		ld 	a, #1 ;Restart of cont_bounce
		ld 	(_cont_bounce), a

		ld	a, #0 ;Se resetea la id del golpe
		ld	(_id_shot), a
	ret

update_sign_player::
	call	entityman_getSignArray

	ld	a, (_score_player)
   	cp	#0
   	jr	z, score_pl_00

	cp	#1
	jr	z, score_pl_15

	cp	#2
	jr	z, score_pl_30

	ld	hl, #_sp_marc_40
	ld 	s_sp_l(ix), l
     	ld 	s_sp_h(ix), h
     	jr	end_upt_sign_player

     	score_pl_00:
     	ld	hl, #_sp_marc_00
	ld 	s_sp_l(ix), l
     	ld 	s_sp_h(ix), h
     	jr	end_upt_sign_player

	score_pl_15:
	ld	hl, #_sp_marc_15
	ld 	s_sp_l(ix), l
     	ld 	s_sp_h(ix), h
     	jr	end_upt_sign_player

     	score_pl_30:
	ld	hl, #_sp_marc_30
	ld 	s_sp_l(ix), l
     	ld 	s_sp_h(ix), h	

	end_upt_sign_player:
	call entityman_getSignArray
	call rendersys_render_signs ;Renderizo el nuevo cartel de puntuacion

	ret

update_sign_cpu::
	call	entityman_getSignArray
	;Paso al segundo cartel que es el de la cpu
	ld bc, #sizeof_s
   	add ix, bc

   	ld	a, (_score_cpu)
   	cp	#0
   	jr	z, score_cpu_00

	cp	#1
	jr	z, score_cpu_15

	cp	#2
	jr	z, score_cpu_30

	ld	hl, #_sp_marc_40
	ld 	s_sp_l(ix), l
     	ld 	s_sp_h(ix), h
     	jr	end_upt_sign_cpu

     	score_cpu_00:
     	ld	hl, #_sp_marc_00
	ld 	s_sp_l(ix), l
     	ld 	s_sp_h(ix), h
     	jr	end_upt_sign_cpu

	score_cpu_15:
	ld	hl, #_sp_marc_15
	ld 	s_sp_l(ix), l
     	ld 	s_sp_h(ix), h
     	jr	end_upt_sign_cpu

     	score_cpu_30:
	ld	hl, #_sp_marc_30
	ld 	s_sp_l(ix), l
     	ld 	s_sp_h(ix), h	

	end_upt_sign_cpu:
	call entityman_getSignArray
	call rendersys_render_signs ;Renderizo el nuevo cartel de puntuacion

	ret