;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Retro Tennis Super Wave 
;;  Copyright (C) 2019 Bamboo Studio 
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  Bamboo Studio.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see http://www.gnu.org/licenses/.
;;------------------------------------------------------------------------------
;;
;; RENDER SYSTEM
;;
.include "cpct_functions.h.s"
.include "man/entity_manager.h.s"

rendersys_init::

   ld    c, #0 ; MODE 1
   call  cpct_setVideoMode_asm ;Sets the video mode
   ld    hl, #_palette ;Palette array
   ld    de, #16 ;Size of the palette
   call  cpct_setPalette_asm ;Set palette
   ;cpctm_setBorder_asm HW_WHITE ;Set border color

   ;Unzip background
   ld    hl, #_mygraphics_end
   ld    de, #0xFFFF
   call  cpct_zx7b_decrunch_s_asm

   ret

rendersys_render_signs::
   ld a, #2
   ld (_sign_counter), a

_signloop:
   ld    de, #0xC000 ;; DE = Pointer to start of the screen
   ld    c, s_x(ix) ;; x
   ld    b, s_y(ix) ;; y
   call  cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL
   
   ex    de, hl
   ld    l,  s_sp_l(ix) ;; Pointer to sprite low byte
   ld    h,  s_sp_h(ix) ;; Pointer to sprite high byte
   ld    c, s_w(ix) ;Width
   ld    b, s_h(ix) ;Height
   call cpct_drawSprite_asm ;Draw sprite 

   _sign_counter = .+1
   ld  a, #0
   dec a
   ret z

   ld (_sign_counter), a
   ld bc, #sizeof_s
   add ix, bc
   jr _signloop

   ret

rendersys_render_ee::
   ld bc, #sizeof_s
   add ix, bc
   add ix, bc

   ld    de, #0xC000 ;; DE = Pointer to start of the screen
   ld    c, s_x(ix) ;; x
   ld    b, s_y(ix) ;; y
   call  cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL
   
   ex    de, hl
   ld    l,  s_sp_l(ix) ;; Pointer to sprite low byte
   ld    h,  s_sp_h(ix) ;; Pointer to sprite high byte
   ld    c, s_w(ix) ;Width
   ld    b, s_h(ix) ;Height
   call cpct_drawSprite_asm ;Draw sprite 

   ret

;;INPUT
;; IX: Pointer to first entity to render
;; A: Number of entities to render
rendersys_update::
   ;Render entities
   call sys_eren_render_entities
   ret

sys_eren_render_entities::
   ld (_ent_counter), a


_renloop:

   ;Erase previous instance
   ld   e, e_lastVP_l(ix) ;Pointer to last position
   ld   d, e_lastVP_h(ix)
   ld l,  e_sp_l(ix) ;; Pointer to sprite low byte
   ld h,  e_sp_h(ix) ;; Pointer to sprite high byte
   ld   c, e_h(ix) ;Height
   ld   b, e_w(ix) ;Width
   push bc ;Width and height stored in the stack 
   call cpct_drawSpriteBlended_asm ;Draw sprite again to erase trace left

   ;; Calculate a video-memory location for printing a string
   ld   de, #0xC000 ;; DE = Pointer to start of the screen
   ld    c, e_x(ix) ;; x
   ld    b, e_y(ix) ;; y
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL


   ld e_lastVP_l(ix), l
   ld e_lastVP_h(ix), h

   ex de, hl ;DE = Pointer to memory
   ld l,  e_sp_l(ix) ;; Pointer to sprite low byte
   ld h,  e_sp_h(ix) ;; Pointer to sprite high byte
   pop bc ;Width and height
   call cpct_drawSpriteBlended_asm
   

_ent_counter = .+1
   ld  a, #0
   dec a
   ret z

   ld (_ent_counter), a
   ld bc, #sizeof_e
   add ix, bc
   jr _renloop

   ret